/*
 * Copyright (c) 2007 NTT DATA Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.terasoluna.fw.batch.ibatissupport;


import jp.terasoluna.fw.batch.core.AbstractCollector;
import jp.terasoluna.fw.batch.core.CollectedDataHandler;
import jp.terasoluna.fw.batch.core.CollectorResult;
import jp.terasoluna.fw.batch.core.JobStatus;
import jp.terasoluna.fw.batch.core.QueueingException;
import jp.terasoluna.fw.batch.openapi.JobContext;
import jp.terasoluna.fw.batch.openapi.ReturnCode;

import org.springframework.orm.ibatis.SqlMapClientTemplate;
import org.springframework.orm.ibatis.support.SqlMapClientDaoSupport;

import com.ibatis.sqlmap.client.event.RowHandler;

/**
 * iBatispΏۃf[^擾̎NXB
 *
 */
public class IBatisDbCollectorImpl extends AbstractCollector<JobContext> {

    /**
     * Ώۃf[^擾pSQLL[B
     */
        private String sql = null;

    /**
     * <code>SqlMapClient</code>f[^ANZX<code>Object</code>B
     */
    private SqlMapClientDaoSupport queryDAO = null;

    /**
     * Ώۃf[^擾s郁\bhB
     *
     * @param jobContext WuReLXg
     * @param collectedDataHandler [JL[
     * @param jobStatus 
     * @return RN^
     */
    @Override
    protected CollectorResult doCollect(JobContext jobContext,
            CollectedDataHandler collectedDataHandler, JobStatus jobStatus) {
        SqlMapClientTemplate sqlMapClientTemplate = 
            queryDAO.getSqlMapClientTemplate();
        QueuePutRowHandler handler = 
            new QueuePutRowHandler(collectedDataHandler, jobStatus);
        
        try {
            sqlMapClientTemplate.queryWithRowHandler(sql, jobContext, handler);
        } catch (RuntimeException e) {
            if (e.getCause() != null && e.getCause().getCause() != null
                    && (e.getCause().getCause() instanceof QueueingException)
                    && !jobStatus.isExecuting()) {
                throw new QueueingException(e);
            } else {
                throw e;
            }
        }
        return 
            new CollectorResult(ReturnCode.NORMAL_END, handler.getCollected());
    }


    /**
     * <code>Row</code>nh̎NXB
     *
     */
    private static class QueuePutRowHandler implements RowHandler {

        /**
         * Ώۃf[^擾B
         */
        private int collected = 0;

        /**
         * Ώۃf[^擾nhB
         */
        private CollectedDataHandler collectedDataHandler;

        /**
         * 󋵁B
         */
        private JobStatus jobStatus;

        /**
         * RXgN^B
         *
         * @param collectedDataHandler Ώۃf[^擾i[郏[JL[
         * @param jobStatus 
         */
        public QueuePutRowHandler(CollectedDataHandler collectedDataHandler,
                JobStatus jobStatus) {
            this.collectedDataHandler = collectedDataHandler;
            this.jobStatus = jobStatus;
        }

        /**
         * <code>Row</code>nh\bhB
         * DB擾<code>Row</code>f[^L[Ɋi[B
         *
         * @param row DB擾<code>Row</code>
         */
        public void handleRow(Object row) {
            collectedDataHandler.handle(row, collected++);
            jobStatus.incrementCollected();
        }

        /**
         * Ώۃf[^擾擾B
         *
         * @return Ώۃf[^擾
         */
        public int getCollected() {
            return collected;
        }
    }

    /**
     * <code>SqlMapClient</code>f[^ANZX<code>Object</code>ݒ肷B
     *
     * @param queryDAO <code>SqlMapClient</code>f[^ANZX<code>Object</code>
     */
    public void setQueryDAO(SqlMapClientDaoSupport queryDAO) {
        this.queryDAO = queryDAO;
    }

    /**
     * Ώۃf[^擾pSQLL[ݒ肷B
     *
     * @param sql Ώۃf[^擾pSQLL[
     */
    public void setSql(String sql) {
        this.sql = sql;
    }
}
